#include "p33FJ256MC710.h"
#include <math.h>
#include "pwm.h"
#include "uart.h"
#include "adc.h"
#include "timer.h"                                               

/****************************** Peripherals Configuration *********************************/

/*-> ADC1 <-*/
void ADC1_config(void)
{
		unsigned int con1,con2,con3,con4,conL,conH,scanL,scanH;
		con1 = 	ADC_MODULE_OFF & ADC_IDLE_CONTINUE 
				& ADC_AD12B_10BIT &
				ADC_FORMAT_INTG & ADC_CLK_TMR & ADC_AUTO_SAMPLING_ON 
				& ADC_SAMP_ON;// & ADC_SIMULTANEOUS;
		con2 = 	ADC_VREF_AVDD_AVSS & ADC_SCAN_OFF & ADC_SELECT_CHAN_0 
				& ADC_ALT_BUF_OFF & ADC_ALT_INPUT_OFF & ADC_DMA_ADD_INC_1;
		con3 =	ADC_CONV_CLK_5Tcy & ADC_SAMPLE_TIME_31;				
		con4 = 	ADC_DMA_BUF_LOC_1;
		conL = 	ENABLE_AN2_ANA;// & ENABLE_AN2_ANA;
		conH =	ENABLE_ALL_DIG_16_31;
		scanL = SCAN_NONE_0_15;
		scanH =	SCAN_NONE_16_31;
       	
		OpenADC1(con1, con2, con3, con4, conL, conH, scanH, scanL); //-> OFF.
		
		SetChanADC1(ADC_CH123_POS_SAMPLEA_3_4_5 & ADC_CH123_NEG_SAMPLEA_VREFN, 
		ADC_CH0_NEG_SAMPLEA_VREFN & ADC_CH0_POS_SAMPLEA_AN2);
		
		IFS0bits.AD1IF = 0;      /* Clear the A/D Interrupt flag bit or else the CPU will
                                	keep vectoring back to the ISR*/           
       	ConfigIntADC1(ADC_INT_PRI_6 & ADC_INT_ENABLE);    /* This function clears the Interrupt 
                                                          Flag (ADIF) bit and then sets the 
                                                          interrupt priority and enables/disables 
                                                          the interrupt. */  
		// AD1CON3bits.SAMC = th;
		AD1CON3bits.ADCS = 3;	// Error in the file "adc.h" -> ADCS=4 then Tad=5Tcy...
		AD1CON3bits.ADRC = 0;     
       return;      
}

/*-> PWM <-*/
void PWM_config(unsigned int dc, unsigned int Ts )
{
           /****************** PWM CONFIGURATION ********************/  
  	unsigned int config;         /* Holds the PWM interrupt configuration value */
  	unsigned int period;         /* Holds the value to be loaded into dutycycle register */
  	unsigned int sptime;         /* Holds the value to be loaded into special event compare register */
  	unsigned int config1;        /* Holds PWM configuration value */
  	unsigned int config2;        /* Holds the value be loaded into PWMCON1 register */
  	unsigned int config3;        /* Holds the value to configure the special event trigger postscale 
                                  and dutycycle */
  	unsigned int dutycyclereg;   /* The value of dutycyclereg determines the duty cycle register
                                  (PDCx) to be written */
  	unsigned int dutycycle;
  	unsigned char updatedisable;
  
  	/* This function, ConfigIntMCPWM( config ), clears the Interrupt Flag bit, sets the 
     interrupt priority and enables/disables the interrupt.*/
  	config = (PWM_INT_DIS & PWM_FLTA_DIS_INT & PWM_INT_PR5 & PWM_FLTA_INT_PR0);
  	IFS3bits.PWMIF = 0;
  	ConfigIntMCPWM( config );
  
  	/* Configure PWM to generate square wave of 9.8% duty cycle */
  	/* This function, SetDCMCPWM(dutycyclereg,dutycycle,updatedisable), configures the 
     Duty Cycle register and updates the PWM Update Disable bit in the PWMCON2 register.*/
  	dutycyclereg = 3;
  	dutycycle = dc;    /* N.B. il duty cycle va scritto nei bit <15:1> e il primo bit serve
                          per aumentare la risoluzione del pwm. ->  come fare << 1. */
  	updatedisable = 0;   /* No duty cycle changes or period value changes will have effect 
                          while UDIS = 1.*/
  	SetDCMCPWM(dutycyclereg,dutycycle,updatedisable);
  
  	OVDCONbits.POVD2H=0; // (SM Hbridge-control) pin is controlled by the value in the corresponding POUTxH:POUTxL bit. 
  	OVDCONbits.POUT2H=0; // (SM Hbridge-control) pin is driven inactive when this bit is cleared. 
 	//OVDCONbits.POVD2H=1;

  	/* This function configures dead time values and clock prescalers */
  	// SetMCPWMDeadTimeGeneration(PWM_DTAPS8 & PWM_DTA18); /* Fcy=116.8/4MHz, dead_time = 10 usec*/
  
  	/* This function, OpenMCPWM(period,sptime,config1,config2,config3), configure the 
     motor control PWM module. It configures the PTPER, SEVTCMP, PTCON, PWMCON1 and PWMCON2 
     registers. */
  	period = Ts; /* -> 499 -> Tpwm=499+1 -> 500 -> 40usec ovvero f_pwm=25KHz.
                    (The period is: 500 -> because of free MODE) 
                     F_pwm = 25KHz, T_pwm = 40usec, Prescaler = 1 (Period=Fcy/F_pwm/Pre-1)*/
  	sptime = 0x0018; /* Special event time inside the pwm period to start A/D conversion. */
  	config1 = (PWM_DIS & PWM_OP_SCALE1 & PWM_IPCLK_SCALE1 & PWM_MOD_FREE);
  	config2 = (PWM_MOD2_IND & PWM_MOD3_IND & PWM_PDIS3H & PWM_PEN2H & PWM_PDIS1H & // Independent mode pwm1.
  	PWM_PEN3L & PWM_PDIS2L & PWM_PDIS1L & PWM_PDIS4H & PWM_PDIS4L);
  	config3 = (PWM_SEVOPS1 & PWM_OSYNC_PWM & PWM_UEN);
  	OpenMCPWM(period,sptime,config1,config2,config3);
            /**********************************************************/ 
  	return; 
}


/*-> UART1 <-*/
void UART1_Config(unsigned int baudvalue)
{
	unsigned int U1MODEvalue; // Holds the value of uart config reg 
	unsigned int U1STAvalue;  // Holds the information regarding uart TX & RX interrupt modes 
	CloseUART1(); // Turn off UART1 module 
	
	// Configure uart1 receive and transmit interrupt 
	IFS0bits.U1RXIF = 0;
	ConfigIntUART1(UART_RX_INT_EN & UART_RX_INT_PR5 & UART_TX_INT_DIS & UART_TX_INT_PR2);
	
	U1MODEvalue = 	UART_EN & UART_UEN_00 &  UART_IrDA_DISABLE & UART_BRGH_SIXTEEN & UART_UXRX_IDLE_ONE & UART_IDLE_CON & UART_DIS_WAKE
			  		& UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_NO_PAR_8BIT & UART_1STOPBIT;
	U1MODEvalue = U1MODEvalue & 0xFFF9; //becuase the UART_NO_PAR_8BIT mask gives a wrong result
	U1STAvalue = UART_TX_ENABLE  & UART_ADR_DETECT_DIS & UART_SYNC_BREAK_DISABLED & UART_INT_RX_BUF_FUL & UART_IrDA_POL_INV_ZERO;
	OpenUART1(U1MODEvalue, U1STAvalue, baudvalue);
	return;
}

/*-> TIMER23 <-*/
void TIMER23_config(unsigned long int match_value)
{
  	/* Enable Timer45 Interrupt and Priority to "1" 
     This function clears the 16-bit Interrupt Flag (TxIF) bit and then sets the
     interrupt priority and enables/disables the interrupt.*/
  	IFS0bits.T3IF = 0; /* Clear Timer5 interrupt flag */
  	ConfigIntTimer23(T3_INT_PRIOR_6 & T3_INT_OFF);
  
  	/* This function writes the 16-bit value into the Timer register. */
  	WriteTimer23(0);
  	/* This function configures the 16-bit Timer Control register and sets the
     period match value into the PR register. */
  	/* match_value = 0xFFFFFF;  */ 
  	OpenTimer23(T2_OFF & T2_GATE_OFF & T2_PS_1_1 & T2_32BIT_MODE_ON 
              	& T2_SOURCE_INT, match_value);
  	return;
}

/***************************************************************************
* Function Name     : send_by_UART                                         *
* Description       : This function puts the data string to be transmitted *
*                     into the transmit buffer 						       *
* Parameters        : unsigned int * address of the string buffer to be    *  
*                     transmitted                                          *
* Return Value      : None                                                 *
***************************************************************************/

void send_by_UART1(unsigned int *buffer, unsigned int n)
{
    char* temp_ptr = (char*) buffer;
    /* transmit till NULL character is encountered */
    if(U1MODEbits.PDSEL == 3)       	/* check if TX is 8bits or 9bits */
    {
        while(*buffer != '\0') 
        {
            while(U1STAbits.UTXBF); 	/* wait if the buffer is full */
            U1TXREG = *buffer++;    	/* transfer data word to TX reg */
        }
    }
    else
    {
        // while(*temp_ptr != '\0')
        int i;
        for(i=0; i<n; i++)
        {
            while(U1STAbits.UTXBF);  	/* wait if the buffer is full */
            U1TXREG = *temp_ptr++;   	/* transfer data byte to TX reg */
        }
    }
    return;
}
/********************************************************************************/
